//Copyright (c) 2011 eXXcellent solutions GmbH
//
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
//documentation files (the "Software"), to deal in the Software without restriction, including without limitation
//the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
//to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//The above copyright notice and this permission notice shall be included in all copies or substantial portions of
//the Software.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
//TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package de.exxcellent.ant;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.util.FileUtils;

/**
 * Mit diesem Task kann man den ClassPath von and selbst zur Laufzeit erweitern. Das ist an den Stellen notwendig und sinnvoll, wo Ant
 * selbst schon eine Task-Implementierung bereitstellt, aber noch weitere Abhängigkeiten im ANT_HOME/lib erwartet (welche erst mal nicht
 * vorhanden sind). <p/> <p>Beispiele hierfür sind die Ant Tasks &lt;scp&gt; und &lt;ftp&gt;.</p> <p/> <p>Damit man also ein build-File
 * ausliefern kann, welches diese Tasks verwendet aber NICHT VOM BENUTZER verlangt seine lokale Ant-Installation anzupassen, braucht man den
 * extendclasspath Tasks.</p> <p/> <p>Verwendungsbeispiel:</p>
 * <pre>
 * &lt;target name="-init.ftptask" unless="ftptask.initialized"&gt;
 *   &lt;taskdef resource="de/exxcellent/ant/tasks.properties" classpath="tools/ant-colony/ant-clover/etc/ant-exxtasks.jar"/&gt;
 *   &lt;extendclasspath path="${dir.tools}/ant-optional/commons-net-1.4.1.jar"/&gt;
 *   &lt;extendclasspath path="${dir.tools}/ant-optional/jakarta-oro-2.0.8.jar"/&gt;
 *   &lt;available property="ftptask.initialized" classname="org.apache.commons.net.ftp.FTPClientConfig"/&gt;
 * &lt;/target&gt;
 * </pre>
 */
public class ExtendClassPath extends Task {

    private final Collection paths = new ArrayList();
    private final Method addURLMethod;

    public ExtendClassPath() {
        try {
            this.addURLMethod = java.net.URLClassLoader.class.getDeclaredMethod("addURL", new Class[] {URL.class});
        } catch (NoSuchMethodException e) {
            throw new BuildException("Unable to setup classpath extender", e);
        }
        this.addURLMethod.setAccessible(true);
    }

    private void addURL(URLClassLoader ucl, URL path) throws NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        this.addURLMethod.invoke(ucl, new Object[] {path});
    }

    public void execute() {
        ClassLoader loader = getProject().getClass().getClassLoader();
        while (!(loader instanceof URLClassLoader)) {
            loader = loader.getParent();
            if (loader == null) {
                throw new BuildException("Unable to find a URLClassLoader to which path may be added");
            }
        }

        try {
            Iterator iterator;
            for (iterator = this.paths.iterator(); iterator.hasNext(); ) {
                Path path = (Path) iterator.next();
                String[] pathElements = path.list();
                for (int i = 0; i < pathElements.length; i++) {
                    String pathElement = pathElements[i];
                    URL url = new URL(FileUtils.getFileUtils().toURI(pathElement));
                    addURL((URLClassLoader) loader, url);
                }
            }
        } catch (Exception e) {
            throw new BuildException("Unable to add paths to the Classloader", e);
        }
    }

    public void addPath(Path path) {
        this.paths.add(path);
    }

    public void setPath(String path) {
        this.paths.add(new Path(getProject(), path));
    }
}